/*
	*
	* This file is a part of CoreFM.
	* A file manager for CuboCore Application Suite
    * Copyright 2019 CuboCore Group
	*

	*
	* This program is free software; you can redistribute it and/or modify
	* it under the terms of the GNU General Public License as published by
	* the Free Software Foundation; either version 3 of the License, or
	* (at your option) any later version.
	*

	*
	* This program is distributed in the hope that it will be useful,
	* but WITHOUT ANY WARRANTY; without even the implied warranty of
	* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	* GNU General Public License for more details.
	*

	*
	* You should have received a copy of the GNU General Public License
	* along with this program; if not, write to the Free Software
	* Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
	* MA 02110-1301, USA.
	*
*/


// Local Headers
#include "corefm.hpp"
#include "nodedialog.hpp"

// Init moveItems
bool CoreFM::moveItems = false;

CoreFM::CoreFM( QString path ) : QMainWindow() {

	// qDebug() << QTime::currentTime() << "CoreFM Starting Up... ";

	mSzRunner = new SizeRunner();
	// qDebug() << QTime::currentTime() << "SizeRunner()";

	loadSettings();
	// qDebug() << QTime::currentTime() << "loadSettings()";

    startSetup();
	setWindowProperties();
	// qDebug() << QTime::currentTime() << "setWindowProperties()";
	makeConnections();
	// qDebug() << QTime::currentTime() << "makeConnections()";

	view1->loadAddress( path );
	// qDebug() << QTime::currentTime() << "view1->loadAddress()";
	view2->loadAddress( startupPath );
	// qDebug() << QTime::currentTime() << "view2->loadAddress()";

	view1->activateWindow();
	// qDebug() << QTime::currentTime() << "view1->activateWindow()";
}

void CoreFM::loadSettings()
{
	touchMode = sm->value( "CoreApps", "TouchMode" );
	startupPath = QString(sm->value( "CoreFM", "StartupPath" ));
	showHidden = sm->value( "CoreFM", "ShowHidden" );
	showThumb = sm->value( "CoreFM", "ShowThumb" );
    viewMode = static_cast<int>( sm->value( "CoreFM", "ViewMode" ));
	zoomIconSize = sm->value( "CoreFM", "ZoomIcon" );
};

void CoreFM::startSetup() {

	// qDebug() << QTime::currentTime() << "Building UI";

	addressBar = new AddressBar( this );
	// qDebug() << QTime::currentTime() << "AddressBar";

	// qDebug() << QTime::currentTime() << "View1";
	view1 = new FolderView( this );

	// qDebug() << QTime::currentTime() << "View2";
	view2 = new FolderView( this );

	createToolBar();
	// qDebug() << QTime::currentTime() << "ToolBar";

	QSplitter *splitter = new QSplitter( Qt::Horizontal, this );
	splitter->setHandleWidth( 2 );
	splitter->setStyleSheet( "QSplitter::handle{image: url(:/icons/splitter.png);}" );
	// qDebug() << QTime::currentTime() << "Splitter";

	splitter->addWidget( view1 );
	splitter->addWidget( view2 );
	// qDebug() << QTime::currentTime() << "Add view1 and view2 to splitter";

	view2->hide();

	sideView = new SideView( this );
	// qDebug() << QTime::currentTime() << "SideView";

	QGridLayout *lyt = new QGridLayout();
	lyt->setContentsMargins( QMargins() );
	lyt->setSpacing( 0 );

	lyt->addWidget( sideView, 0, 0, 2, 1 );
	lyt->addWidget( addressBar, 0, 1 );
	lyt->addWidget( splitter, 1, 1 );
	// qDebug() << QTime::currentTime() << "Layout";

	QWidget *base = new QWidget( this );
	base->setLayout( lyt );
	// qDebug() << QTime::currentTime() << "Base";

	setCentralWidget( base );
	// qDebug() << QTime::currentTime() << "setCentralWidget";
};

void CoreFM::createToolBar() {

    newFolderAct = new QAction( QIcon::fromTheme( "folder-new" ), "New Folder", this );
	newFolderAct->setShortcut( tr( "F10" ) );
	connect( newFolderAct, SIGNAL( triggered() ), this, SLOT( newFolder() ) );

	newFileAct = new QAction( QIcon::fromTheme( "document-new" ), "New Text", this );
	newFileAct->setShortcut( tr( "Shift+F10" ) );
	connect( newFileAct, SIGNAL( triggered() ), this, SLOT( newFile() ) );

	showHiddenAct = new QAction( QIcon::fromTheme( "password-show-on" ), "Hidden", this );
	showHiddenAct->setShortcut( tr( "Ctrl+H" ) );
	connect( showHiddenAct, SIGNAL( triggered() ), this, SLOT( toggleHidden() ) );
	showHiddenAct->setCheckable( true );
	showHiddenAct->setChecked( showHidden );

	previewAct = new QAction( QIcon::fromTheme( "bitmap-trace" ), "Toggle Previews", this );
	previewAct->setShortcut( tr( "Ctrl+Shift+I" ) );
	connect( previewAct, SIGNAL( triggered() ), this, SLOT( togglePreviews() ) );
	previewAct->setCheckable( true );
	previewAct->setChecked( showThumb );

    termAct = new QAction( QIcon::fromTheme( "utilities-terminal" ), "Open Terminal Here", this );
	termAct->setShortcut( tr( "F4" ) );
	connect( termAct, SIGNAL( triggered() ), this, SLOT( openTerminal() ) );

	cutAct = new QAction( QIcon::fromTheme( "edit-cut" ), "Cut", this );
	cutAct->setShortcut( tr( "Ctrl+X" ) );
	connect( cutAct, SIGNAL( triggered() ), this, SLOT( prepareMove() ) );

	copyAct = new QAction( QIcon::fromTheme( "edit-copy" ), "Copy", this );
	copyAct->setShortcut( tr( "Ctrl+C" ) );
	connect( copyAct, SIGNAL( triggered() ), this, SLOT( prepareCopy() ) );

	pasteAct = new QAction( QIcon::fromTheme( "edit-paste" ), "Paste", this );
	pasteAct->setShortcut( tr( "Ctrl+V" ) );
	connect( pasteAct, SIGNAL( triggered() ), this, SLOT( paste() ) );

	openAct = new QAction( QIcon::fromTheme( "document-open" ), "Open", this );
	openAct->setShortcuts( QList<QKeySequence>() << tr( "Enter" ) << tr( "Return" ) );
	connect( openAct, SIGNAL( triggered() ), this, SLOT( loadAddress() ) );

	refreshAct = new QAction( QIcon::fromTheme( "view-refresh" ), "Refresh", this );
	refreshAct->setShortcut( tr( "F5" ) );
	connect( refreshAct, SIGNAL( triggered() ), this, SLOT( refresh() ) );

	selectAllAct = new QAction( QIcon::fromTheme( "edit-select-all" ), "Select All", this );
	selectAllAct->setShortcut( tr( "Ctrl+A" ) );
	connect( selectAllAct, SIGNAL( triggered() ), this, SLOT( selectAll() ) );

	renameAct = new QAction( QIcon::fromTheme( "edit-rename" ), "Rename", this );
	renameAct->setShortcut( tr( "F2" ) );
	connect( renameAct, SIGNAL( triggered() ), this, SLOT( rename() ) );

	batchRenameAct = new QAction( QIcon::fromTheme( "edit-rename" ), "Batch Rename", this );
	batchRenameAct->setShortcut( tr( "Shift+F2" ) );
    connect( batchRenameAct, SIGNAL( triggered() ), this, SLOT( openRenamer() ) );

	trashAct = new QAction( QIcon::fromTheme( "edit-delete" ), "Move to Trash", this );
	trashAct->setShortcut( tr( "Delete" ) );
	connect( trashAct, SIGNAL( triggered() ), this, SLOT( trashSelected() ) );

	deleteAct = new QAction( QIcon::fromTheme( "edit-delete" ) , "Delete", this );
	deleteAct->setShortcut( tr( "Shift+Delete" ) );
	connect( deleteAct, SIGNAL( triggered() ), this, SLOT( deleteSelected() ) );

    propsAct = new QAction( QIcon::fromTheme( "document-properties" ), "Properties", this );
	propsAct->setShortcut( tr( "Alt+Return" ) );
	connect( propsAct, SIGNAL( triggered() ), this, SLOT( showProperties() ) );

	compressAct = new QAction( QIcon::fromTheme( "archive-insert" ), "Compress", this );
	// compressAct->setShortcut( tr( "" ) );
	connect( compressAct, SIGNAL( triggered() ), this, SLOT( compress() ) );

	extractAct = new QAction( QIcon::fromTheme( "archive-extract" ), "Extract", this );
	// extractAct->setShortcut( tr( "" ) );
	connect( extractAct, SIGNAL( triggered() ), this, SLOT( extract() ) );

    backAct = new QAction( QIcon::fromTheme( "go-next" ), "Previous", this );
	backAct->setShortcut( tr( "Alt+Left" ) );
    connect( backAct, SIGNAL( triggered() ), this, SLOT( goBack() ) );

    openWithAct = new QAction( QIcon::fromTheme( "quickopen-file" ), "Open With", this );
    // openWithAct->setShortcut( tr( "" ) );
    connect( openWithAct, SIGNAL( triggered() ), this, SLOT( selectApp() ) );

    upAct = new QAction( QIcon::fromTheme( "go-previous" ), "Parent dir", this );
    upAct->setShortcut( tr( "Backspace" ) );
	connect( upAct, SIGNAL( triggered() ), this, SLOT( goUp() ) );

//	forwardAct = new QAction( QIcon::fromTheme( "go-next" ), "Forward", this );
//	forwardAct->setShortcut( tr( "Alt+Right" ) );
//	connect( forwardAct, SIGNAL( triggered() ), this, SLOT( goForward() ) );

	splitAct = new QAction( QIcon::fromTheme( "view-split-left-right" ), "Toggle Split View", this );
	connect( splitAct, SIGNAL( triggered() ), this, SLOT( toggleSplit() ) );
	splitAct->setCheckable( true );
	splitAct->setChecked( false );

    viewChangeAct = new QAction( QIcon::fromTheme( "view-list" ), "Change View", this );
	viewChangeAct->setShortcut( tr( "Ctrl+Shift+V" ) );
	connect( viewChangeAct, SIGNAL( triggered() ), this, SLOT( nextView() ) );
	switch( viewMode ) {
		case 0: {
            viewChangeAct->setIcon( QIcon::fromTheme( "view-grid" ) );
			break;
		}
		case 1: {
            viewChangeAct->setIcon( QIcon::fromTheme( "view-list" ) );
			break;
		}
		case 2: {
            viewChangeAct->setIcon( QIcon::fromTheme( "view-more" ) );
			break;
		}
	}

	newWindowAct = new QAction( QIcon::fromTheme( "window-new" ), "New Window", this );
	// extractAct->setShortcut( tr( "" ) );
	connect( newWindowAct, SIGNAL( triggered() ), this, SLOT( newWindow() ) );

	sortNameAct = new QAction( "Name", this );
	sortNameAct->setCheckable( true );
	sortNameAct->setChecked( true );

	sortSizeAct = new QAction( "Size", this );
	sortSizeAct->setCheckable( true );
	sortSizeAct->setChecked( false );

	sortDateAct = new QAction( "Date", this );
	sortSizeAct->setCheckable( true );
	sortSizeAct->setChecked( false );

	sortOrderAct = new QAction( "Ascending", this );
	sortOrderAct->setCheckable( true );
	sortOrderAct->setChecked( true );

	QActionGroup *sortGroup = new QActionGroup( this );
	sortGroup->setExclusive( true );
	sortGroup->addAction( sortNameAct );
	sortGroup->addAction( sortSizeAct );
	sortGroup->addAction( sortDateAct );

	connect( sortGroup, SIGNAL( triggered( QAction * ) ), this, SLOT( setSortType( QAction * ) ) );
	connect( sortOrderAct, SIGNAL( triggered() ), this, SLOT( setSortOrder() ) );

	treeWriteAct = new QAction( QIcon::fromTheme( "document-new" ), "Save Dir Tree as text", this );
	connect( treeWriteAct, SIGNAL( triggered() ), this, SLOT( writeTree() ) );

	metadataAct = new QAction( QIcon::fromTheme( "document-properties" ), "Open Metadata viewer", this );
	connect( metadataAct, SIGNAL( triggered() ), this, SLOT( openMetadataViewer() ) );

	searchAct = new QAction( QIcon::fromTheme( "edit-find" ), "Search", this );
	connect( searchAct, SIGNAL( triggered() ), this, SLOT( openSearch() ) );

	zoomInAct = new QAction( QIcon::fromTheme( "zoom-in" ), "Zoom In", this );
	zoomInAct->setShortcut( tr( "Ctrl++" ) );
	connect( zoomInAct, SIGNAL( triggered() ), this, SLOT( zoomIn() ) );
	view1->addAction( zoomInAct );
	view2->addAction( zoomInAct );

	zoomOutAct = new QAction( QIcon::fromTheme( "zoom-out" ), "Zoom Out", this );
	zoomOutAct->setShortcut( tr( "Ctrl+-" ) );
	connect( zoomOutAct, SIGNAL( triggered() ), this, SLOT( zoomOut() ) );
	view1->addAction( zoomOutAct );
	view2->addAction( zoomOutAct );

	addressBar->setBackAction( backAct );
	addressBar->setUpAction( upAct );
//    addressBar->setForwardAction( forwardAct );
	addressBar->setIOActions( cutAct, copyAct, pasteAct );
	addressBar->setPropsAction( propsAct );
	addressBar->setFindAction( searchAct );
	addressBar->setSelectAllAction( selectAllAct );


	addressBar->addAction( newFolderAct );
	addressBar->addAction( newFileAct );
	addressBar->addAction( renameAct );
    addressBar->addAction( openWithAct );

	addressBar->addAction( showHiddenAct );
	addressBar->addAction( previewAct );

	addressBar->addAction( termAct );

	addressBar->addAction( splitAct );

	addressBar->addAction( zoomInAct );
	addressBar->addAction( zoomOutAct );
	addressBar->addAction( viewChangeAct );

	addAction( newFolderAct );
	addAction( newFileAct );
	addAction( showHiddenAct );
	addAction( previewAct );
	addAction( termAct );
	addAction( cutAct );
	addAction( copyAct );
	addAction( pasteAct );
	addAction( openAct );
	addAction( refreshAct );
	addAction( selectAllAct );
	addAction( renameAct );
	addAction( trashAct );
	addAction( deleteAct );
	addAction( propsAct );
	addAction( batchRenameAct );
	addAction( compressAct );
    addAction( extractAct );
	addAction( newWindowAct );
	addAction( sortNameAct );
	addAction( sortSizeAct );
	addAction( sortDateAct );
	addAction( sortOrderAct );
	addAction( backAct );
	addAction( upAct );
//	addAction( forwardAct );
	addAction( splitAct );
	addAction( viewChangeAct );
    addAction( treeWriteAct );
	addAction( metadataAct );
	addAction( searchAct );
	addAction( zoomInAct );
	addAction( zoomOutAct );
};

void CoreFM::makeConnections() {

	connect( view1, SIGNAL( contextMenu( const QPoint & ) ), this, SLOT( contextMenu( const QPoint & ) ) );
	connect( view2, SIGNAL( contextMenu( const QPoint & ) ), this, SLOT( contextMenu( const QPoint & ) ) );

	connect( view1, SIGNAL( updateVarious() ), this, SLOT( updateVarious() ) );
	connect( view2, SIGNAL( updateVarious() ), this, SLOT( updateVarious() ) );

	connect( view1, SIGNAL( openFile( QString ) ), this, SLOT( openInApp( QString ) ) );
	connect( view2, SIGNAL( openFile( QString ) ), this, SLOT( openInApp( QString ) ) );

	connect( view1, &FolderView::updateAddress, addressBar, &AddressBar::setText );
	connect( view2, &FolderView::updateAddress, addressBar, &AddressBar::setText );

	connect( sideView, SIGNAL( changeDir( QString ) ), this, SLOT( loadAddress( QString ) ) );
	connect( sideView, SIGNAL( pinIt() ), this, SLOT( pinIt() ) );
	connect( sideView, SIGNAL( shareIt() ), this, SLOT( shareIt() ) );
	connect( sideView, SIGNAL( showTrash() ), this, SLOT( showTrash() ) );
	connect( sideView, SIGNAL( toggleTools() ), addressBar, SLOT( toggleTools() ) );

	connect( mSzRunner, SIGNAL( currentCount( quint64, quint64, quint64 ) ), addressBar, SLOT( setSize( quint64, quint64, quint64 ) ) );
	connect( mSzRunner, SIGNAL( finalCount( quint64, quint64, quint64 ) ), addressBar, SLOT( setSize( quint64, quint64, quint64 ) ) );
};

void CoreFM::setWindowProperties() {

    setWindowTitle( "CoreFM " );
	setWindowIcon( QIcon( ":/corefm.svg" ) );

	if (touchMode) {
		setWindowFlags( windowFlags() );
		setWindowState( windowState() | Qt::WindowMaximized | Qt::WindowActive );
	}

	else
		resize( 800, 500 );
};

FolderView* CoreFM::currentView() {

	if ( not view2->isVisible() )
		return view1;

	if ( view1->hasFocus() )
		return view1;

	else
		return view2;
};

void CoreFM::newFolder() {

	QString path = currentView()->rootPath();

	NodeDialog *dlg = new NodeDialog( NodeDialog::NewFolder, path, QString(), this );
	dlg->exec();
};

void CoreFM::newFile() {

	QString path = currentView()->rootPath();

	NodeDialog *dlg = new NodeDialog( NodeDialog::NewFile, path, QString(), this );
	dlg->exec();
};

void CoreFM::toggleHidden() {

	sm->setValue( "CoreFM", "ShowHidden", not showHidden );
	currentView()->refresh();
};

void CoreFM::togglePreviews() {

	sm->setValue( "CoreFM", "ShowThumb", not showThumb );
};

void CoreFM::openTerminal() {

	QStringList selectedList = currentView()->selection();

	if ( selectedList.count() ) {
		QFileInfo file(selectedList.value( 0 ));
		if(file.isDir()){
            CPrime::AppOpenFunc::defaultAppEngine( CPrime::Category::Terminal, selectedList.value( 0 ), "", this );
			return;
		} else{
            CPrime::InfoFunc::messageEngine("CoreFM", "corefm", "Can't open terminal here", "Please select a folder to open terminal there.", this );
			return;
		}
	}

    CPrime::AppOpenFunc::defaultAppEngine( CPrime::Category::Terminal, currentView()->rootPath(), "", this );
};

void CoreFM::prepareMove() {

	QStringList selectedItems = currentView()->selection();
	if ( not selectedItems.count() )
		return;

	moveItems = true;

	QList<QUrl> urlList;
	foreach( QString item, selectedItems )
		urlList << QUrl::fromLocalFile( item );

	QMimeData *mData = new QMimeData();
	mData->setUrls( urlList );

	clipBoard->setMimeData( mData );

	currentView()->clearSelection();
};

void CoreFM::prepareCopy() {

	if ( not currentView()->selection().count() )
		return;

	moveItems = false;

	QList<QUrl> urlList;
	foreach( QString item, currentView()->selection() )
		urlList << QUrl::fromLocalFile( item );

	QMimeData *mData = new QMimeData();
	mData->setUrls( urlList );

	clipBoard->setMimeData( mData );

	currentView()->clearSelection();
};

void CoreFM::paste() {

	const QMimeData *mimeData = clipBoard->mimeData();

	if ( mimeData->hasUrls() ) {
		CoreProcess::Process *process = new CoreProcess::Process;
		process->sourceDir = CPrime::FileUtils::dirName( mimeData->urls().at( 0 ).toLocalFile() );
		process->targetDir = currentView()->rootPath();

		if ( not process->sourceDir.endsWith( "/" ) )
			process->sourceDir += "/";

		QStringList srcList;
		foreach( QUrl url, mimeData->urls() ) {
			QString file = url.toLocalFile();
			if ( CPrime::FileUtils::exists( file ) )
				srcList << file.replace( process->sourceDir, "" );
		}

		if ( not srcList.count() )
			return;

		if ( moveItems )
			process->type = CoreProcess::Move;

		else
			process->type = CoreProcess::Copy;

		IODialog *pasteDlg = new IODialog( srcList, process );
		pasteDlg->show();
	}

	clipBoard->setMimeData( new QMimeData() );
	pasteAct->setVisible( false );
};

void CoreFM::refresh() {

	if ( view1->hasFocus() )
		view1->refresh();

	else
		view2->refresh();
};

void CoreFM::selectAll() {

	currentView()->selectAll();
};

void CoreFM::rename() {

	QString path;
	QStringList selection;

	NodeDialog *dlg = new NodeDialog( NodeDialog::Rename, currentView()->rootPath(), currentView()->selection().value( 0 ), this );
	dlg->exec();
};

void CoreFM::openRenamer() {

    CPrime::AppOpenFunc::defaultAppEngine( CPrime::Category::Renamer, currentView()->selection().value( 0 ), "", this );
};

void CoreFM::openMetadataViewer() {

    CPrime::AppOpenFunc::defaultAppEngine( CPrime::Category::MetadataViewer, currentView()->selection().value( 0 ), "", this );
}

void CoreFM::trashSelected() {

	CPrime::TrashManager::moveToTrash( currentView()->selection() );
};

void CoreFM::deleteSelected() {

	CPrime::TrashManager::deleteFileTotally( currentView()->selection(), false );
};

void CoreFM::showProperties() {

	QString currentFile;

	if ( currentView()->selection().count() )
		currentFile = currentView()->selection().value( 0 );

	else
		currentFile = currentView()->rootPath();

	propertiesw *props = new propertiesw( currentFile, this );
	props->show();
};

void CoreFM::compress() {

    CPrime::AppOpenFunc::defaultAppEngine( CPrime::Category::Archiver, currentView()->selection().value( 0 ), "", this );
};

void CoreFM::extract() {

    CPrime::AppOpenFunc::defaultAppEngine( CPrime::Category::Archiver, currentView()->selection().value( 0 ), "", this );
};

void CoreFM::goBack() {

	currentView()->goBack();
};

void CoreFM::goForward() {

	currentView()->goForward();
};

void CoreFM::goUp() {

	currentView()->goUp();
};

void CoreFM::openSearch() {

	QStringList selectedList = currentView()->selection();

	if ( selectedList.count() ) {
		QFileInfo file(selectedList.value( 0 ));
		if ( file.isDir() )
            CPrime::AppOpenFunc::defaultAppEngine( CPrime::Category::SearchApp, selectedList.value( 0 ), "", this );

		else
            CPrime::AppOpenFunc::defaultAppEngine( CPrime::Category::SearchApp, CPrime::FileUtils::dirName( selectedList.value( 0 ) ), "", this );

		return;
	}

    CPrime::AppOpenFunc::defaultAppEngine( CPrime::Category::SearchApp, currentView()->rootPath(), "", this );
};

void CoreFM::writeTree() {

	QString fileName = QFileDialog::getSaveFileName(
		this,
		"CoreFM - Save Directory Tree As",
		currentView()->rootPath(),
		"All Files (*.*)"
	);

	if ( not fileName.count() )
		return;

	QString path;
	if ( not currentView()->selection().count() )
		path = currentView()->rootPath();

	else
		path = currentView()->selection().value( 0 );

	TreeWriter *writer = new TreeWriter();
	writer->writeTree( path, fileName );
};

void CoreFM::nextView() {

	viewMode = ( viewMode + 1 ) % 3;
	sm->setValue( "CoreFM", "ViewMode", viewMode );

	/* IconView */
	if ( viewMode == 0 )
		viewChangeAct->setIcon( QIcon::fromTheme( "view-list-icons" ));

	/* TileView */
	else if ( viewMode == 1 )
		viewChangeAct->setIcon( QIcon::fromTheme( "view-list-details" ) );

	/* DetailsView */
	else
		viewChangeAct->setIcon( QIcon::fromTheme( "view-list-text" ) );

	view1->refreshView();
	view2->refreshView();
};

void CoreFM::updateVarious() {

	addressBar->setText( currentView()->rootPath() );
	setWindowTitle( "CoreFM - " + CPrime::FileUtils::baseName( currentView()->rootPath() ) );

	addressBar->setFreeSpace( QString( "%1 free" ).arg( CPrime::FileFunc::formatSize( QStorageInfo( currentView()->rootPath() ).bytesAvailable() ) ) );

	QStringList selected = currentView()->selection();

	/* Some selection */
	if ( selected.count() ) {
		if ( mSzRunner->isRunning() )
			mSzRunner->terminate();

		addressBar->setSize( "Counting..." );
		mSzRunner->setPaths( selected );
		mSzRunner->start();

		cutAct->setVisible( true );
		copyAct->setVisible( true );

		if ( selected.count() > 1 ) {
			renameAct->setVisible( false );
			propsAct->setVisible( false );
		}

		else {
			renameAct->setVisible( true );
			propsAct->setVisible( true );
            openWithAct->setVisible(true);
		}
	}

	/* No selection */
	else {
		cutAct->setVisible( false );
		copyAct->setVisible( false );

		renameAct->setVisible( false );
        openWithAct->setVisible(false);
		propsAct->setVisible( true );

		/* The subtraction of 2 is to account for . and .. */
		struct dirent **fileList;
		quint64 count = scandir( currentView()->rootPath().toStdString().c_str(), &fileList, nullptr, nullptr ) - 2;


		if ( count )
			addressBar->setSize( QString( "%1 item%2" ).arg( count ).arg( count > 1 ? "s" : "" ) );

		else
			addressBar->setSize( "Empty" );
	}

	if ( currentView()->canGoBack() )
		backAct->setEnabled( true );

	else
		backAct->setDisabled( true );

//	if ( currentView()->canGoForward() )
//		forwardAct->setEnabled( true );

//	else
//		forwardAct->setDisabled( true );

	/* Clipboard */
	if ( clipBoard->mimeData()->hasUrls() )
		pasteAct->setVisible( true );

	else
		pasteAct->setVisible( false );

	/* Up action */
	if ( currentView()->canGoUp() )
		upAct->setEnabled( true );

	else
		upAct->setDisabled( true );
};

void CoreFM::openInApp( QString path ) {

	if ( path.count() ) {
		CPrime::AppOpenFunc::appOpenEngine( path, this );
        return;
	}

	QAction *action = dynamic_cast<QAction *>( sender() );

	if ( action ) {
		CoreDesktopFile app = action->data().value<CoreDesktopFile>();
		app.startApplicationWithArgs( currentView()->selection() );
	}
};

void CoreFM::selectApp() {

	// Select application in the dialog
	ApplicationDialog *dialog = new ApplicationDialog( this );

	if ( dialog->exec() ) {
		if ( dialog->getCurrentLauncher().compare( "" ) != 0 ) {
			CPrime::AppOpenFunc::systemAppOpener( dialog->getCurrentLauncher(), QStringList() << currentView()->selection().value( 0 ) );

			// Function from utilities.cpp
			CPrime::InfoFunc::saveToRecent( dialog->getCurrentLauncher(), currentView()->selection().value( 0 ) );
		}
	}
};

void CoreFM::setSortType( QAction *act ) {

	QString text = act->text();
	bool asc = sortOrderAct->isChecked();

	if ( text == "Name" ) {
		view1->sort( 0, ( asc ? Qt::AscendingOrder : Qt::DescendingOrder ) );
		view2->sort( 0, ( asc ? Qt::AscendingOrder : Qt::DescendingOrder ) );
	}

	else if ( text == "Size" ) {
		view1->sort( 1, ( asc ? Qt::AscendingOrder : Qt::DescendingOrder ) );
		view2->sort( 1, ( asc ? Qt::AscendingOrder : Qt::DescendingOrder ) );
	}

	else if ( text == "Date" ) {
		view1->sort( 3, ( asc ? Qt::AscendingOrder : Qt::DescendingOrder ) );
		view2->sort( 3, ( asc ? Qt::AscendingOrder : Qt::DescendingOrder ) );
	}
};

void CoreFM::setSortOrder() {

	bool asc = sortOrderAct->isChecked();

	if ( sortNameAct->isChecked() ) {
		view1->sort( 0, ( asc ? Qt::AscendingOrder : Qt::DescendingOrder ) );
		view2->sort( 0, ( asc ? Qt::AscendingOrder : Qt::DescendingOrder ) );
	}

	else if ( sortSizeAct->isChecked() ) {
		view1->sort( 1, ( asc ? Qt::AscendingOrder : Qt::DescendingOrder ) );
		view2->sort( 1, ( asc ? Qt::AscendingOrder : Qt::DescendingOrder ) );
	}

	else if ( sortDateAct->isChecked() ) {
		view1->sort( 3, ( asc ? Qt::AscendingOrder : Qt::DescendingOrder ) );
		view2->sort( 3, ( asc ? Qt::AscendingOrder : Qt::DescendingOrder ) );
	}
};

void CoreFM::zoomIn() {

	zoomIconSize = zoomIconSize + 8;

	if ( zoomIconSize > 128 )
		zoomIconSize = 128;

	view1->setIconSize( zoomIconSize );
	view2->setIconSize( zoomIconSize );

	sm->setValue( "CoreFM", "ZoomIcon", zoomIconSize );
};

void CoreFM::zoomOut() {

	zoomIconSize = zoomIconSize - 8;

	if ( zoomIconSize < 16 )
		zoomIconSize = 16;

    view1->setIconSize( zoomIconSize );
    view2->setIconSize( zoomIconSize );

	sm->setValue( "CoreFM", "ZoomIcon", zoomIconSize);
};

void CoreFM::newWindow( QString path ) {

	CoreFM *cfm = new CoreFM( path );

	if (touchMode)
		cfm->showMaximized();

	else
		cfm->show();
};

void CoreFM::loadAddress( QString path ) {

	QAction *openAct = qobject_cast<QAction *>( sender() );

	if ( openAct )
		currentView()->loadAddress( currentView()->selection().value( 0 ) );

	else
		currentView()->loadAddress( path );
};

void CoreFM::showTrash() {

	TrashView *trash = new TrashView();

	trash->setWindowTitle( "CoreFM - Trash" );
	trash->setWindowIcon( QIcon( ":/corefm.svg" ) );

	trash->setAttribute( Qt::WA_DeleteOnClose );

	if ( touchMode )
		trash->showMaximized();

	else
		trash->show();
};

void CoreFM::toggleSplit() {

	if ( splitAct->isChecked() ) {
		view2->show();
		view1->activateWindow();
	}

	else {
		view2->hide();
		view1->activateWindow();
	}
};

void CoreFM::pinIt() {

	QStringList selectedItems = currentView()->selection();

	if ( selectedItems.count() ) {
		Q_FOREACH( QString path, selectedItems )
			pinit::callPinIt( this, path );
	}

	else {

		 pinit::callPinIt( this, currentView()->rootPath() );
	}
};

void CoreFM::shareIt() {

    if (currentView()->selection().count()) {
        shareit::shareFiles( this, currentView()->selection() );
    } else {
        CPrime::InfoFunc::messageEngine("CoreFM", "corefm", "Warning!!!", "Nothing Selected.\nSelect some files to share.", this);
    }
};
