/*
	*
	* This file is a part of CoreFM.
	* A file manager for CuboCore Application Suite
    * Copyright 2019 CuboCore Group
	*

	*
	* This program is free software; you can redistribute it and/or modify
	* it under the terms of the GNU General Public License as published by
	* the Free Software Foundation; either version 3 of the License, or
	* (at your option) any later version.
	*

	*
	* This program is distributed in the hope that it will be useful,
	* but WITHOUT ANY WARRANTY; without even the implied warranty of
	* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	* GNU General Public License for more details.
	*

	*
	* You should have received a copy of the GNU General Public License
	* along with this program; if not, write to the Free Software
	* Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
	* MA 02110-1301, USA.
	*
*/


#include "nodedialog.hpp"


NodeDialog::NodeDialog( NodeDialog::Mode mode, QDir curDir, QString text, QWidget *parent ) : QDialog( parent ) {

	dlgMode = mode;

	QVBoxLayout *lyt = new QVBoxLayout();
	QHBoxLayout *btnLyt = new QHBoxLayout();

	dir = QDir( curDir );
	dir.makeAbsolute();

	if ( !text.isNull() )
		data = QString( text );

	QLabel *lbl = new QLabel();
	le = new QLineEdit( this );

	okBtn = new QPushButton( QIcon::fromTheme( "dialog-apply" ), "Create", this );
	connect( okBtn, SIGNAL( clicked() ), this, SLOT( accept() ) );

	cancelBtn = new QPushButton( QIcon::fromTheme( "dialog-close" ), "Cancel", this );
	connect( cancelBtn, SIGNAL( clicked() ), this, SLOT( cancel() ) );

	if ( mode == NodeDialog::NewFolder ) {
		setWindowTitle( "CoreFM - New Folder" );

		lbl->setText( tr( "Create a new folder in <b>%1</b> named:" ).arg( curDir.path() ) );
		lbl->setWordWrap( true );
		le->setText( "New Folder" );
		le->selectAll();
	}

	else if ( mode == NodeDialog::NewFile ) {
		setWindowTitle( "CoreFM - New File" );

		lbl->setText( tr( "Create a new file in <b>%1</b> named:" ).arg( curDir.path() ) );
		lbl->setWordWrap( true );
		le->setText( "New File" );
		le->selectAll();
	}

	else {
		setWindowTitle( "CoreFM - Rename" );

		QMimeType mt = mimeDb.mimeTypeForFile( curDir.filePath( text ) );
		QFileInfo info( curDir.filePath( text ) );

		okBtn->setText( "Rename" );

		lbl->setText( tr( "Rename <b>%1</b> in the folder in <b>%2</b> named:" ).arg( text ).arg( curDir.path() ) );
		lbl->setWordWrap( true );
		le->setText( info.completeBaseName() + ( info.isDir() ? "" : "." + info.suffix() ) );
		le->setSelection( 0, info.completeBaseName().size() );
	}

	btnLyt->addStretch( 0 );
	btnLyt->addWidget( okBtn );
	btnLyt->addWidget( cancelBtn );

	lyt->addWidget( lbl );
	lyt->addWidget( le );
	lyt->addLayout( btnLyt );

	connect( le, SIGNAL( textEdited( QString ) ), this, SLOT( handleTextChanged( QString ) ) );
	connect( le, SIGNAL( returnPressed() ), this, SLOT( accept() ) );

	setLayout( lyt );
	le->setFocus();

	handleTextChanged( le->text() );
	setMinimumWidth( 450 );
};

QString NodeDialog::name() {

	return le->text();
}

void NodeDialog::cancel() {

	le->clear();
	reject();
};

void NodeDialog::handleTextChanged( QString newText ) {

	if ( newText.isEmpty() ) {
		okBtn->setDisabled( true );
		return;
	}

	if ( dir.entryList().contains( newText ) )
		okBtn->setDisabled( true );

	else
		okBtn->setEnabled( true );
};

void NodeDialog::accept() {

	switch( dlgMode ) {
		case NodeDialog::NewFolder: {
			if ( not dir.mkdir( le->text() ) )
                CPrime::InfoFunc::messageEngine("CoreFM", "corefm", "Can't create new folder", "Unable to create new folder at <b>" + dir.path() + "</b>", this );

			break;
		}

		case NodeDialog::NewFile: {
			QFile file( dir.filePath( le->text() ) );
			if ( not file.open( QFile::WriteOnly ) )
                CPrime::InfoFunc::messageEngine("CoreFM", "corefm", "Can't create new file", "Unable to create new file at <b>" + dir.path() + "</b>", this );

			file.close();

			break;
		}

		case NodeDialog::Rename: {
			QString source = dir.filePath( data );
			QString target = dir.filePath( le->text() );
			if ( not QFile::rename( source, target ) )
                CPrime::InfoFunc::messageEngine("CoreFM", "corefm", "Can't rename", "Unable to rename <b>" + data + "</b>", this );

			break;
		}
	}

	QDialog::accept();
};
