/*
	*
	* This file is a part of CoreFM.
	* A file manager for CuboCore Application Suite
    * Copyright 2019 CuboCore Group
	*

	*
	* This program is free software; you can redistribute it and/or modify
	* it under the terms of the GNU General Public License as published by
	* the Free Software Foundation; either version 3 of the License, or
	* (at your option) any later version.
	*

	*
	* This program is distributed in the hope that it will be useful,
	* but WITHOUT ANY WARRANTY; without even the implied warranty of
	* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	* GNU General Public License for more details.
	*

	*
	* You should have received a copy of the GNU General Public License
	* along with this program; if not, write to the Free Software
	* Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
	* MA 02110-1301, USA.
	*
*/

#include <QtConcurrent>
#include "sideview.hpp"

SideView::SideView( QWidget *parent ) : QWidget( parent ) {

    touchMode = sm->value( "CoreApps", "TouchMode" );
    sideviewSize = sm->value( "CoreApps", "SideViewSize" );
    sideviewIconSize = sm->value( "CoreApps", "SideViewIconSize" );

	createUI();

	QTimer::singleShot( 100, this, &SideView::populateSideView );
	QTimer::singleShot( 200, this, &SideView::repopulateDrives );

    if ( touchMode )
		toggleCompact();
};

void SideView::createUI() {

    setFixedWidth( sideviewSize );
    setContentsMargins( QMargins() );

	title = new QToolButton( this );
    title->setIcon( QIcon( ":/corefm.svg" ) );
    title->setText( " CoreFM" );
    title->setFont( QFont( font().family(), 16, QFont::Bold ) );
	title->setToolButtonStyle( Qt::ToolButtonTextBesideIcon );
	title->setSizePolicy( QSizePolicy::MinimumExpanding, QSizePolicy::Fixed );
    title->setIconSize( sideviewIconSize );
	title->setAutoRaise( true );
	title->setFocusPolicy( Qt::NoFocus );

	connect( title, SIGNAL( clicked() ), this, SLOT( toggleCompact() ) );

	itemsWidget = new QWidget();
    itemsWidget->setContentsMargins( QMargins() );

	drivesWidget = new QWidget();
    drivesWidget->setContentsMargins( QMargins() );

	stack = new QStackedWidget();
    stack->setSizePolicy( QSizePolicy::MinimumExpanding, QSizePolicy::Preferred );
	stack->addWidget( itemsWidget );
	stack->addWidget( drivesWidget );
	stack->setCurrentIndex( 0 );

	QVBoxLayout *wLyt = new QVBoxLayout();
	wLyt->setContentsMargins( 0, 20, 0, 10 );
	wLyt->setSpacing( 0 );

	wLyt->addWidget( title );
	wLyt->addWidget( stack );

	QWidget *base = new QWidget();
	base->setLayout( wLyt );

	QVBoxLayout *lyt = new QVBoxLayout();
	lyt->setContentsMargins( QMargins() );
	lyt->setSpacing( 0 );
	lyt->addWidget( base );

	setLayout( lyt );
	setStyleSheet( CPrime::ThemeFunc::getSideViewStyleSheet() );
}

void SideView::populateSideView() {

	qDeleteAll( itemsWidget->children() );

	/* Update devices list */
	sMgr = StorageManager::instance();
	connect( sMgr, SIGNAL( deviceAdded( QString ) ), this, SLOT( repopulateDrives() ) );
	connect( sMgr, SIGNAL( deviceRemoved( QString ) ), this, SLOT( repopulateDrives() ) );

	QVBoxLayout *itemsLyt = new QVBoxLayout();
	itemsLyt->setContentsMargins( QMargins( 0, 20, 0, 10 ) );
	itemsLyt->setSpacing( 0 );

	// Home
    SideViewItem *homeItem = new SideViewItem( "Home", QIcon::fromTheme( "user-home" ), QDir::homePath(), false, this );
	connect( homeItem, SIGNAL( clicked( QString ) ), this, SIGNAL( changeDir( QString ) ) );
	itemsLyt->addWidget( homeItem );

	// Desktop
	if ( CPrime::FileUtils::exists( QDir::home().filePath( "Desktop" ) ) ) {
		SideViewItem *dktpItem = new SideViewItem( "Desktop", QIcon::fromTheme( "user-desktop" ), QDir::home().filePath( "Desktop" ), false, this );
		connect( dktpItem, SIGNAL( clicked( QString ) ), this, SIGNAL( changeDir( QString ) ) );
		itemsLyt->addWidget( dktpItem );
	}

    // Home folders
    QStringList folders = QStringList() << "Downloads";
    Q_FOREACH( QString path, folders ) {
        if ( CPrime::FileUtils::exists( QDir::home().filePath( path ) ) ) {
            SideViewItem *item = new SideViewItem( path, QIcon::fromTheme( QString( "folder-%1" ).arg( path.toLower() ) ), QDir::home().filePath( path ), false, this );
            connect( item, SIGNAL( clicked( QString ) ), this, SIGNAL( changeDir( QString ) ) );
            itemsLyt->addWidget( item );
        }
    }

	// File system
	SideViewItem *rootItem = new SideViewItem( "FileSystem", QIcon::fromTheme( "folder-root" ), "/", false, this );
	connect( rootItem, SIGNAL( clicked( QString ) ), this, SIGNAL( changeDir( QString ) ) );
	itemsLyt->addWidget( rootItem );

	// Drives
    SideViewItem *driveItem = new SideViewItem( "Drives", QIcon::fromTheme( "drive-multidisk" ), "", false, this );
	connect( driveItem, SIGNAL( clicked() ), this, SLOT( switchWidget()	 ) );
	itemsLyt->addWidget( driveItem );

	// Trash
    SideViewItem *trashItem = new SideViewItem( "Trash", QIcon::fromTheme( "user-trash" ), "trash://", false, this );
	connect( trashItem, SIGNAL( clicked() ), this, SIGNAL( showTrash() ) );
	itemsLyt->addWidget( trashItem );

	itemsLyt->addStretch();

	// Pin It
    SideViewItem *shareitItem = new SideViewItem( "Share It", QIcon::fromTheme( "document-send" ), "", false, this );
    connect( shareitItem, SIGNAL( clicked() ), this, SIGNAL( shareIt() ) );
    itemsLyt->addWidget( shareitItem );

    SideViewItem *pinitItem = new SideViewItem( "Pin It", QIcon::fromTheme( "bookmark-new" ), "", false, this );
    connect( pinitItem, SIGNAL( clicked() ), this, SIGNAL( pinIt() ) );
    itemsLyt->addWidget( pinitItem );

    SideViewItem *toolsItem = new SideViewItem( "Tools", QIcon::fromTheme( "tools-wizard" ), "", false, this );
	connect( toolsItem, SIGNAL( clicked() ), this, SIGNAL( toggleTools() ) );
    itemsLyt->addWidget( toolsItem );

	itemsWidget->setLayout( itemsLyt );
};

void SideView::repopulateDrives() {

	qDeleteAll( drivesWidget->children() );

	QVBoxLayout *drivesLyt = new QVBoxLayout();
	drivesLyt->setContentsMargins( QMargins() );
	drivesLyt->setSpacing( 0 );

	QWidget *spacer = new QWidget();
    spacer->setFixedSize( 10, 20 );
	drivesLyt->addWidget( spacer );

    SideViewItem *backBtn = new SideViewItem( "Back", QIcon::fromTheme( "go-previous" ), "", false, this );
	connect( backBtn, SIGNAL( clicked() ), this, SLOT( switchWidget() ) );

	drivesLyt->addWidget( backBtn, Qt::AlignCenter );
	drivesLyt->addWidget( new QLabel( "   " ) );			/* Small spacer between back button and other drives */

	Q_FOREACH( StorageBlock dev, sMgr->validBlocks() ) {

		SideViewItem *item;
		if ( dev.isOptical() )
			item = new SideViewItem( dev.label(), QIcon::fromTheme( "drive-optical" ), dev.mountPoint(), true, this );

		else if ( dev.isRemovable() )
			item = new SideViewItem( dev.label(), QIcon::fromTheme( "drive-removable-media-usb" ), dev.mountPoint(), true, this );

		else {
			if ( dev.mountPoint() == "/" )
				item = new SideViewItem( dev.label(), QIcon::fromTheme( "drive-harddisk-root" ), dev.mountPoint(), true, this );

			else
				item = new SideViewItem( dev.label(), QIcon::fromTheme( "drive-harddisk" ), dev.mountPoint(), true, this );
		}

		item->setDevice( dev );
		connect( item, SIGNAL( clicked( QString ) ), this, SIGNAL( changeDir( QString ) ) );
		drivesLyt->addWidget( item );
    }

	drivesLyt->addStretch();
	drivesWidget->setLayout( drivesLyt );
};

void SideView::switchWidget() {

	short int curIdx = stack->currentIndex();

	if ( curIdx == 1 )
		stack->setCurrentIndex( 0 );

	else
		stack->setCurrentIndex( 1 );
};

void SideView::toggleCompact() {

    if ( touchMode ) {
		/* Switch to compact mode */
		mCompact = not mCompact;

		if ( mCompact ) {
            setFixedWidth(sideviewIconSize.width() + 10);
			title->setToolButtonStyle( Qt::ToolButtonIconOnly );
		}

		else {
            setFixedWidth( sideviewSize );
			title->setToolButtonStyle( Qt::ToolButtonTextBesideIcon );
		}

		Q_FOREACH( SideViewItem *item, findChildren<SideViewItem *>() )
			item->toggleWidth();

		return;
	}
};

void SideView::paintEvent( QPaintEvent *pEvent ) {

	QPainter painter( this );
	painter.setPen( Qt::NoPen );

	painter.setBrush( QColor( 0, 0, 0, 20 ) );
	painter.drawRect( rect() );

	painter.end();

	QWidget::paintEvent( pEvent );

	pEvent->accept();
};

/*
	*
	* SideView - SidePanelItem class for CoreFM
	*
*/

SideViewItem::SideViewItem( QString name, QIcon icon, QString target, bool drive, QWidget *parent ) : QWidget( parent ) {

    mIconSize = QSize( sm->value( "CoreApps", "SideViewIconSize" ) ).width();

	setFixedHeight( mIconSize + 8 );
	setSizePolicy( QSizePolicy::MinimumExpanding, QSizePolicy::Fixed );

	mName = name;
	mIcon = icon;
	mTarget = target;
	isDrive = drive;

	if ( CPrime::FileUtils::isDir( mTarget ) and not mTarget.endsWith( "/" ) )
		mTarget += "/";

	mHover = false;
	mHoverMount = false;
	mPressed = false;

	if ( not drive )
		setToolTip( mTarget );

	setCursor( Qt::PointingHandCursor );
	setAcceptDrops( true );

	setMouseTracking( true );
};

void SideViewItem::setDevice( StorageBlock dev ) {

	if ( not isDrive )
		return;

	/* Toggle mount button */
	mntBtn = new QToolButton();
	mntBtn->setFixedSize( QSize( mIconSize + 8, mIconSize + 8 ) );
	mntBtn->setIconSize( QSize( mIconSize - 4, mIconSize - 4 ) );
	mntBtn->setAutoRaise( true );
	mntBtn->setFocusPolicy( Qt::NoFocus );

	/* Mount this drive */
	if ( dev.mountPoint().isEmpty() ) {
		mntBtn->setIcon( QIcon::fromTheme( "emblem-mounted" ) );
		mntBtn->setToolTip( "Click to mount" );
	}

	else{
		mntBtn->setIcon( QIcon::fromTheme( "emblem-unmounted" ) );
		mntBtn->setToolTip( "Click to unmount" );
	}

	QHBoxLayout *lyt = new QHBoxLayout();
	lyt->setContentsMargins( QMargins() );
	lyt->addStretch();
	lyt->addWidget( mntBtn );
	setLayout( lyt );

	connect( mntBtn, SIGNAL( clicked() ), this, SLOT( toggleMount() ) );

	mDev = dev;
};

void SideViewItem::toggleWidth() {

//    compact = not compact;
	if ( isDrive ) {
		if ( compact )
			mntBtn->hide();

		else
			mntBtn->show();
	}

	repaint();
};

void SideViewItem::dragEnterEvent( QDragEnterEvent *deEvent ) {

	deEvent->acceptProposedAction();
};

void SideViewItem::dragMoveEvent( QDragMoveEvent *dmEvent ) {

	const QMimeData *mData = dmEvent->mimeData();
	if ( not mData->hasUrls() ) {
		dmEvent->ignore();
		return;
	}

	/* If the target is not writable, or not trash, ignore the drag/drop */
	if ( not CPrime::FileUtils::isWritable( mTarget ) and not mTarget.startsWith( "trash://" ) ) {
		dmEvent->ignore();
		return;
	}

	/* If the source and target are the same, ignore the drag/drop */
	else if ( CPrime::FileUtils::dirName( dmEvent->mimeData()->urls().at( 0 ).toLocalFile() ) == mTarget ) {
		dmEvent->ignore();
		return;
	}

	/* All good, do drag/drop */
	else {
		/* Trashing the nodes */
		if ( mTarget == "trash://" ) {
			dmEvent->setDropAction( Qt::MoveAction );
			dmEvent->accept();
		}

		/* Copy the files */
		else {
			dmEvent->setDropAction( Qt::CopyAction );
			dmEvent->accept();
		}
	}
};

void SideViewItem::dropEvent( QDropEvent *dpEvent ) {

	if ( not dpEvent->mimeData()->hasUrls() ) {
		dpEvent->ignore();
		return;
	}

	if ( not CPrime::FileUtils::isWritable( mTarget ) and not mTarget.startsWith( "trash://" ) ) {
		dpEvent->ignore();
		return;
	}

	if ( CPrime::FileUtils::dirName( dpEvent->mimeData()->urls().at( 0 ).toLocalFile() ) == mTarget ) {
		dpEvent->ignore();
		return;
	}

	const QMimeData *mData = dpEvent->mimeData();

	/* Trash the urls */
	if ( mTarget == "trash://" ) {
		QStringList toBeDeleted;
		Q_FOREACH( QUrl url, mData->urls() ) {
			qDebug() << url.toLocalFile();
			toBeDeleted << url.toLocalFile();
		}

		CPrime::TrashManager::moveToTrash( toBeDeleted );
	}

	else {

		qDebug() << "Copying nodes to" << mTarget;

		CoreProcess::Process *process = new CoreProcess::Process;
		process->sourceDir = CPrime::FileUtils::dirName( mData->urls().at( 0 ).toLocalFile() );
		process->targetDir = mTarget;

		if ( not process->sourceDir.endsWith( "/" ) )
			process->sourceDir += "/";

		QStringList srcList;
		Q_FOREACH( QUrl url, mData->urls() ) {
			QString file = url.toLocalFile();
			if ( CPrime::FileUtils::exists( file ) )
				srcList << file.replace( process->sourceDir, "" );
		}

		if ( not srcList.count() )
			return;

		process->type = CoreProcess::Copy;

		IODialog *pasteDlg = new IODialog( srcList, process );
		pasteDlg->show();
	}

	dpEvent->accept();

};

void SideViewItem::mousePressEvent( QMouseEvent *mpEvent ) {

	if ( mpEvent->button() == Qt::LeftButton )
		mPressed = true;

	else if ( mpEvent->button() == Qt::RightButton ) {

		/* IGNORE THIS PART */

		// QMenu menu;
		// menu.setAttribute( Qt::WA_DeleteOnClose );
		// if ( mType == Bookmark ) {
			// menu.addAction( QIcon( ":/icons/delete.png" ), "&Remove Bookmark", this, SLOT( removeBookmark() ) );
			// menu.exec( mapToGlobal( mpEvent->pos() ) );
		// }

		// else if ( mType == QuickFile ) {
			// menu.addAction( QIcon( ":/icons/delete.png" ), "&Remove from Quick Files", this, SLOT( removeQuickFile() ) );
			// menu.exec( mapToGlobal( mpEvent->pos() ) );
		// }
	}

	repaint();
	mpEvent->accept();
};

void SideViewItem::mouseReleaseEvent( QMouseEvent *mrEvent ) {

	if ( mrEvent->button() == Qt::LeftButton ) {
		emit clicked( mTarget );
		emit clicked();
	}

	mPressed = false;

	repaint();
	mrEvent->accept();
};

void SideViewItem::paintEvent( QPaintEvent *pEvent ) {

	QPainter painter( this );
	painter.setPen( Qt::NoPen );

	painter.save();
	if ( mPressed ) {

		painter.setBrush( palette().color( QPalette::Highlight ).darker( 120 ) );
		painter.drawRect( QRect( 0, 0, width(), height() ) );
	}

	/* Hover and no highlight */
	else if ( mHover ) {

		painter.setBrush( palette().color( QPalette::Highlight ).lighter( 110 ) );
		painter.drawRect( QRect( 0, 0, width(), height() ) );
	}

	/* Clear */
	else {

		painter.setBrush( Qt::transparent );
		painter.drawRect( QRect( 0, 0, width(), height() ) );
	}
	painter.restore();

	/* Paint Icon */
	QRect iconRect = QRect( 4, 4, mIconSize, mIconSize );
	painter.drawPixmap( iconRect, mIcon.pixmap( mIconSize ) );

	if ( not compact ) {
		QRectF textRect;
		if ( isDrive )
			textRect = QRectF( mIconSize + 8 + 4, 0, width() - 2 * ( mIconSize + 8 ) + 4, mIconSize + 8 );

		else
			textRect = QRectF( mIconSize + 8 + 4, 0, width() - mIconSize + 8 + 4, mIconSize + 8 );

		painter.save();
		QFontMetrics fm( font() );
		QString text = fm.elidedText( mName, Qt::ElideRight, textRect.width() );

		painter.setPen( palette().color( QPalette::Text ) );

		painter.drawText( textRect, Qt::AlignLeft | Qt::AlignVCenter, text );
		painter.restore();
	}

	painter.end();

	pEvent->accept();
};

void SideViewItem::toggleMount() {

	if ( not isDrive )
		return;

	if ( mDev.mountPoint().count() ) {
		if ( mDev.unmount() ) {
			mDev = StorageBlock( mDev.device().split( "/" ).takeLast() );
			CPrime::InfoFunc::messageEngine(
                "CoreFM", "corefm", "Unmounted",
                QString( "Successfully unmounted %1." ).arg( mDev.device() ),
				this
			);
			mntBtn->setIcon( QIcon::fromTheme( "emblem-mounted" ) );
			mntBtn->setToolTip( "Click to mount" );
		}

		else {
			CPrime::InfoFunc::messageEngine(
                "CoreFM", "corefm", "Unmount failed",
                QString( "Failed to unmount %1." ).arg( mDev.device() ),
				this
			);
		}
	}

	else {
		if ( mDev.mount() ) {
			mDev = StorageBlock( mDev.device().split( "/" ).takeLast() );
			QString mtpt = mDev.mountPoint();
			CPrime::InfoFunc::messageEngine(
                "CoreFM", "corefm", "Mounted",
                QString( "Successfully mounted %1 at %2" ).arg( mDev.device() ).arg( mtpt ),
				this
			);

			mntBtn->setIcon( QIcon::fromTheme( "emblem-unmounted" ) );
			mntBtn->setToolTip( "Click to unmount" );
		}

		else {
			CPrime::InfoFunc::messageEngine(
                "CoreFM", "corefm", "Mount failed",
                QString( "Failed to mount %1." ).arg( mDev.device() ),
				this
			);
		}
	}
};
